@@ -288,26 +288,26 @@ class SeqLinear(nn.Module):
         self.n_heads = n_heads
         self.device = device
         
-        self.alpha = (6 / (self.d_state + self.d_inner)) ** 0.5
         d_in_proj = 2 * self.d_state + self.d_inner + 2 * self.n_heads
         self.in_proj = nn.Linear(d_model, d_in_proj, bias=False, device=device)
+        conv_dim = 2 * self.d_state + self.d_inner
         self.conv = nn.Conv1d(
-            in_channels=d_in_proj,
-            out_channels=d_in_proj,
+            in_channels=conv_dim,
+            out_channels=conv_dim,
             kernel_size=d_conv,
-            groups=d_in_proj,
+            groups=conv_dim,
             padding=d_conv-1,
             device=device,
         )
         self.w_base = nn.Parameter(torch.empty(n_heads, device=device))
-        self.out_proj = nn.Linear(d_inner, d_model, bias=False, device=device)
+        self.out_proj = nn.Linear(self.d_inner, d_model, bias=False, device=device)
         
         self._reset_parameters()
     
     def _reset_parameters(self):
         nn.init.xavier_uniform_(self.in_proj.weight)
         nn.init.xavier_uniform_(self.conv.weight)
-        nn.init.uniform_(self.w_base, -0.1, 0.1)
+        nn.init.uniform_(self.w_base, -0.5, 0.5)
         nn.init.xavier_uniform_(self.out_proj.weight)
         
         if self.conv.bias is not None:
@@ -316,42 +316,51 @@ class SeqLinear(nn.Module):
     def bidirectional(self, x):
         bsz, src_len, d_model = x.size()
         
-        xIOw_f = self.conv(self.in_proj(x).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
-        xIOw_b = self.conv(self.in_proj(x).flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+        xIOwg = self.in_proj(x)
         
-        x_f, I_f, O_f, w_f, g_f = torch.split(
-            xIOw_f,
+        xIO, w, g = torch.split(
+            xIOwg,
             [
-                self.d_state,
-                self.d_state,
-                self.d_inner,
-                self.n_heads,
-                self.n_heads,
-            ],
-            dim=-1
-        )
-        x_b, I_b, O_b, w_b, g_b = torch.split(
-            xIOw_b,
-            [
-                self.d_state,
-                self.d_state,
-                self.d_inner,
+                2 * self.d_state + self.d_inner,
                 self.n_heads,
                 self.n_heads,
             ],
             dim=-1
         )
         
-        x_f = rearrange(self.alpha * x_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        xIO_f = self.conv(xIO.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+        xIO_b = self.conv(xIO.flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+        
+        x_f, I_f, O_f = torch.split(
+            xIO_f,
+            [
+                self.d_state,
+                self.d_state,
+                self.d_inner,
+            ],
+            dim=-1
+        )
+        x_b, I_b, O_b = torch.split(
+            xIO_b,
+            [
+                self.d_state,
+                self.d_state,
+                self.d_inner,
+            ],
+            dim=-1
+        )
+        
+        x_f = rearrange(x_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         I_f = rearrange(I_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         O_f = rearrange(O_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
-        x_b = rearrange(self.alpha * x_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        x_b = rearrange(x_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         I_b = rearrange(I_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         O_b = rearrange(O_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
 
-        decay_f = F.silu(w_f * self.w_base).transpose(1, 2).unsqueeze(-2).repeat(1, 1, src_len, 1)
-        decay_b = F.silu(w_b * self.w_base).transpose(1, 2).unsqueeze(-2).repeat(1, 1, src_len, 1)
+        w = F.softplus(w)
+        decay_f = F.silu(w * self.w_base).transpose(1, 2).unsqueeze(-2).repeat(1, 1, src_len, 1)
+        decay_b = decay_f.flip(1)
         mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=-1)
         decay_f = decay_f.masked_fill(~mask, 0.0)
         decay_b = decay_b.masked_fill(~mask, 0.0)
@@ -365,7 +374,7 @@ class SeqLinear(nn.Module):
         out_f = torch.einsum('bthd, bhts, bshd, bshe -> bthe', x_f, decay_f, I_f, O_f)
         out_b = torch.einsum('bthd, bhts, bshd, bshe -> bthe', x_b, decay_b, I_b, O_b)
         
-        out = out_f * F.sigmoid(g_f).unsqueeze(-1) + out_b * F.sigmoid(g_b).unsqueeze(-1)
+        out = (out_f + out_b) * F.silu(g).unsqueeze(-1)
         out = rearrange(out, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
         return out
@@ -374,21 +383,32 @@ class SeqLinear(nn.Module):
         if mode == "bidirectional": return self.bidirectional(x)
         bsz, src_len, d_model = x.size()
 
-        xIOw = self.conv(self.in_proj(x).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+        xIOwg = self.in_proj(x)
         
-        x, I, O, w, g = torch.split(
-            xIOw,
+        xIO, w, g = torch.split(
+            xIOwg,
             [
-                self.d_state,
-                self.d_state,
-                self.d_inner,
+                2 * self.d_state + self.d_inner,
                 self.n_heads,
                 self.n_heads,
             ],
             dim=-1
         )
         
-        x = rearrange(self.alpha * x, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        xIO = self.conv(xIO.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+        
+        x, I, O = torch.split(
+            xIO,
+            [
+                self.d_state,
+                self.d_state,
+                self.d_inner,
+            ],
+            dim=-1
+        )
+        
+        w = F.softplus(w)
+        x = rearrange(x, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         I = rearrange(I, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         O = rearrange(O, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
@@ -400,18 +420,23 @@ class SeqLinear(nn.Module):
             mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=0)
             decay = decay.masked_fill(~mask, float('-inf'))
             decay = F.softmax(decay, dim=-1)
+            import matplotlib.pyplot as plt
+            plt.clf()
+            plt.plot(decay[0, :, -1].transpose(0, 1).detach().cpu().numpy())
+            plt.pause(0.01)
             out = torch.einsum('bthd, bhts, bshd, bshe -> bthe', x, decay, I, O)
         elif mode == "noncausal":
             decay = F.softmax(-F.silu(w * self.w_base), dim=1)
             out = torch.einsum('bthd, bsh, bshd, bshe -> bthe', x, decay, I, O)
 
-        out = out * F.sigmoid(g).unsqueeze(-1)
+        out = out * F.silu(g).unsqueeze(-1)
         out = rearrange(out, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
         return out
 
 class SeqMLP(nn.Module):
     def __init__(self, emb_dim, input_dim, output_dim,
+                 d_state=None, d_inner=None, d_conv=4,
                  n_layers=1, n_heads=1, dropout=0.0,
                  pos_encoding=False, pos_encoding_max_len=None,
                  use_embedding=True, weight_tying=False,
@@ -441,8 +466,10 @@ class SeqMLP(nn.Module):
             nn.ModuleDict(
                 dict(
                     mixer=SeqLinear(
-                        d_in=emb_dim,
-                        d_out=emb_dim,
+                        d_model=emb_dim,
+                        d_state=d_state,
+                        d_inner=d_inner,
+                        d_conv=d_conv,
                         n_heads=n_heads,
                         device=device
                     ),
